@@ -45,15 +45,17 @@ namespace {
 	auto options = PremiumSubscriptionOptionsFromTL(tlOptions);
 	for (auto i = 0; i < options.size(); i++) {
 		const auto &tlOption = tlOptions[i].data();
+		const auto currency = qs(tlOption.vcurrency());
 		const auto perUserText = Ui::FillAmountAndCurrency(
 			tlOption.vamount().v / float64(tlOption.vusers().v),
-			qs(tlOption.vcurrency()),
+			currency,
 			false);
 		options[i].costPerMonth = perUserText
 			+ ' '
 			+ QChar(0x00D7)
 			+ ' '
 			+ QString::number(tlOption.vusers().v);
+		options[i].currency = currency;
 	}
 	return options;
 }
@@ -613,24 +615,32 @@ std::vector<GiftOptionData> PremiumGiftCodeOptions::optionsForPeer() const {
 	return result;
 }
 
-Data::PremiumSubscriptionOptions PremiumGiftCodeOptions::options(int amount) {
-	const auto it = _subscriptionOptions.find(amount);
+Data::PremiumSubscriptionOptions PremiumGiftCodeOptions::optionsForGiveaway(
+		int usersCount) {
+	const auto skipForStars = [&](Data::PremiumSubscriptionOptions options) {
+		const auto proj = &Data::PremiumSubscriptionOption::currency;
+		options.erase(
+			ranges::remove(options, Ui::kCreditsCurrency, proj),
+			end(options));
+		return options;
+	};
+	const auto it = _subscriptionOptions.find(usersCount);
 	if (it != end(_subscriptionOptions)) {
-		return it->second;
+		return skipForStars(it->second);
 	} else {
 		auto tlOptions = QVector<MTPPremiumGiftCodeOption>();
 		for (auto i = 0; i < _optionsForOnePerson.months.size(); i++) {
 			tlOptions.push_back(MTP_premiumGiftCodeOption(
 				MTP_flags(MTPDpremiumGiftCodeOption::Flags(0)),
-				MTP_int(amount),
+				MTP_int(usersCount),
 				MTP_int(_optionsForOnePerson.months[i]),
 				MTPstring(),
 				MTPint(),
 				MTP_string(_optionsForOnePerson.currencies[i]),
-				MTP_long(_optionsForOnePerson.totalCosts[i] * amount)));
+				MTP_long(_optionsForOnePerson.totalCosts[i] * usersCount)));
 		}
-		_subscriptionOptions[amount] = GiftCodesFromTL(tlOptions);
-		return _subscriptionOptions[amount];
+		_subscriptionOptions[usersCount] = GiftCodesFromTL(tlOptions);
+		return skipForStars(_subscriptionOptions[usersCount]);
 	}
 }
 
@@ -853,6 +863,7 @@ std::optional<Data::StarGift> FromTL(
 			.perUserRemains = data.vper_user_remains().value_or_empty(),
 			.firstSaleDate = data.vfirst_sale_date().value_or_empty(),
 			.lastSaleDate = data.vlast_sale_date().value_or_empty(),
+			.lockedUntilDate = data.vlocked_until_date().value_or_empty(),
 			.requirePremium = data.is_require_premium(),
 			.upgradable = data.vupgrade_stars().has_value(),
 			.birthday = data.is_birthday(),
@@ -880,13 +891,20 @@ std::optional<Data::StarGift> FromTL(
 		const auto releasedById = data.vreleased_by()
 			? peerFromMTP(*data.vreleased_by())
 			: PeerId();
+		const auto themeUserId = data.vtheme_peer()
+			? peerFromMTP(*data.vtheme_peer())
+			: PeerId();
 		const auto releasedBy = releasedById
 			? session->data().peer(releasedById).get()
 			: nullptr;
+		const auto themeUser = themeUserId
+			? session->data().peer(themeUserId).get()
+			: nullptr;
 		auto result = Data::StarGift{
-			.id = uint64(data.vid().v),
+			.id = data.vid().v,
 			.unique = std::make_shared<Data::UniqueGift>(Data::UniqueGift{
 				.id = data.vid().v,
+				.initialGiftId = data.vgift_id().v,
 				.slug = qs(data.vslug()),
 				.title = qs(data.vtitle()),
 				.ownerAddress = qs(data.vowner_address().value_or_empty()),
@@ -895,12 +913,23 @@ std::optional<Data::StarGift> FromTL(
 					? peerFromMTP(*data.vowner_id())
 					: PeerId()),
 				.releasedBy = releasedBy,
+				.themeUser = themeUser,
 				.nanoTonForResale = FindTonForResale(data.vresell_amount()),
 				.starsForResale = FindStarsForResale(data.vresell_amount()),
 				.number = data.vnum().v,
 				.onlyAcceptTon = data.is_resale_ton_only(),
+				.canBeTheme = data.is_theme_available(),
 				.model = *model,
 				.pattern = *pattern,
+				.value = (data.vvalue_amount()
+					? std::make_shared<Data::UniqueGiftValue>(
+						Data::UniqueGiftValue{
+							.currency = qs(
+								data.vvalue_currency().value_or_empty()),
+							.valuePrice = int64(
+								data.vvalue_amount().value_or_empty()),
+						})
+					: nullptr),
 			}),
 			.document = model->document,
 			.releasedBy = releasedBy,
@@ -950,20 +979,20 @@ std::optional<Data::SavedStarGift> FromTL(
 				| ranges::to_vector)
 			: std::vector<int>()),
 		.message = (data.vmessage()
-			? TextWithEntities{
-				.text = qs(data.vmessage()->data().vtext()),
-				.entities = Api::EntitiesFromMTP(
-					session,
-					data.vmessage()->data().ventities().v),
-			}
+			? Api::ParseTextWithEntities(
+				session,
+				*data.vmessage())
 			: TextWithEntities()),
 		.starsConverted = int64(data.vconvert_stars().value_or_empty()),
 		.starsUpgradedBySender = int64(
 			data.vupgrade_stars().value_or_empty()),
+		.giftPrepayUpgradeHash = qs(
+			data.vprepaid_upgrade_hash().value_or_empty()),
 		.fromId = (data.vfrom_id()
 			? peerFromMTP(*data.vfrom_id())
 			: PeerId()),
 		.date = data.vdate().v,
+		.upgradeSeparate = data.is_upgrade_separate(),
 		.upgradable = data.is_can_upgrade(),
 		.anonymous = data.is_name_hidden(),
 		.pinned = data.is_pinned_to_top() && hasUnique,
