@@ -194,16 +194,16 @@ void ShowAddParticipantsError(
 			&& channel->canAddAdmins()) {
 			const auto makeAdmin = [=](Fn<void()> close) {
 				const auto user = forbidden.users.front();
-				const auto weak = std::make_shared<QPointer<EditAdminBox>>();
+				const auto weak = std::make_shared<base::weak_qptr<EditAdminBox>>();
 				const auto done = [=](auto&&...) {
-					if (const auto strong = weak->data()) {
+					if (const auto strong = weak->get()) {
 						strong->uiShow()->showToast(
 							tr::lng_box_done(tr::now));
 						strong->closeBox();
 					}
 				};
 				const auto fail = [=] {
-					if (const auto strong = weak->data()) {
+					if (const auto strong = weak->get()) {
 						strong->closeBox();
 					}
 				};
@@ -446,7 +446,7 @@ void AddContactBox::save() {
 		firstName = lastName;
 		lastName = QString();
 	}
-	const auto weak = Ui::MakeWeak(this);
+	const auto weak = base::make_weak(this);
 	const auto session = _session;
 	_sentName = firstName;
 	_contactId = base::RandomValue<uint64>();
@@ -716,7 +716,7 @@ TimeId GroupInfoBox::ttlPeriod() const {
 }
 
 void GroupInfoBox::createGroup(
-		QPointer<Ui::BoxContent> selectUsersBox,
+		base::weak_qptr<Ui::BoxContent> selectUsersBox,
 		const QString &title,
 		const std::vector<not_null<PeerData*>> &users) {
 	if (_creationRequestId) {
@@ -750,8 +750,8 @@ void GroupInfoBox::createGroup(
 		_creationRequestId = 0;
 		const auto controller = _navigation->parentController();
 		if (type == u"NO_CHAT_TITLE"_q) {
-			const auto weak = Ui::MakeWeak(this);
-			if (const auto strong = selectUsersBox.data()) {
+			const auto weak = base::make_weak(this);
+			if (const auto strong = selectUsersBox.get()) {
 				strong->closeBox();
 			}
 			if (weak) {
@@ -792,10 +792,10 @@ void GroupInfoBox::submit() {
 	} else if (_canAddBot) {
 		createGroup(nullptr, title, { not_null<PeerData*>(_canAddBot) });
 	} else {
-		auto initBox = [title, weak = Ui::MakeWeak(this)](
+		auto initBox = [title, weak = base::make_weak(this)](
 				not_null<PeerListBox*> box) {
 			auto create = [box, title, weak] {
-				if (const auto strong = weak.data()) {
+				if (const auto strong = weak.get()) {
 					strong->createGroup(
 						box.get(),
 						title,
