@@ -15,7 +15,6 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "boxes/background_box.h"
 #include "boxes/stickers_box.h"
 #include "chat_helpers/compose/compose_show.h"
-#include "core/ui_integration.h" // TextContext
 #include "data/stickers/data_custom_emoji.h"
 #include "data/stickers/data_stickers.h"
 #include "data/data_changes.h"
@@ -82,7 +81,6 @@ public:
 		bool selected);
 
 	[[nodiscard]] uint8 index() const;
-	int naturalWidth() const override;
 
 	void setSelected(bool selected);
 
@@ -164,7 +162,6 @@ private:
 	void updateText();
 
 	const uint32 _level;
-	const TextWithEntities _icon;
 	const Ui::Text::MarkedContext _context;
 	Ui::Text::String _text;
 	bool _minimal = false;
@@ -183,6 +180,15 @@ ColorSample::ColorSample(
 		colorIndex
 	) | rpl::start_with_next([=](uint8 index) {
 		_index = index;
+		setNaturalWidth([&] {
+			if (_name.isEmpty() || _style->colorPatternIndex(_index)) {
+				return st::settingsColorSampleSize;
+			}
+			const auto padding = st::settingsColorSamplePadding;
+			return std::max(
+				padding.left() + _name.maxWidth() + padding.right(),
+				padding.top() + st::semiboldFont->height + padding.bottom());
+		}());
 		update();
 	}, lifetime());
 }
@@ -197,6 +203,7 @@ ColorSample::ColorSample(
 , _index(colorIndex)
 , _selected(selected)
 , _simple(true) {
+	setNaturalWidth(st::settingsColorSampleSize);
 }
 
 void ColorSample::setSelected(bool selected) {
@@ -278,16 +285,6 @@ uint8 ColorSample::index() const {
 	return _index;
 }
 
-int ColorSample::naturalWidth() const {
-	if (_name.isEmpty() || _style->colorPatternIndex(_index)) {
-		return st::settingsColorSampleSize;
-	}
-	const auto padding = st::settingsColorSamplePadding;
-	return std::max(
-		padding.left() + _name.maxWidth() + padding.right(),
-		padding.top() + st::semiboldFont->height + padding.bottom());
-}
-
 PreviewWrap::PreviewWrap(
 	not_null<Ui::GenericBox*> box,
 	std::shared_ptr<Ui::ChatStyle> style,
@@ -460,22 +457,12 @@ LevelBadge::LevelBadge(
 	uint32 level,
 	not_null<Main::Session*> session)
 : Ui::RpWidget(parent)
-, _level(level)
-, _icon(Ui::Text::SingleCustomEmoji(
-	session->data().customEmojiManager().registerInternalEmoji(
-		st::settingsLevelBadgeLock,
-		QMargins(0, st::settingsLevelBadgeLockSkip, 0, 0),
-		false)))
-, _context(Core::TextContext({
-	.session = session,
-	.repaint = [this] { update(); },
-})) {
+, _level(level) {
 	updateText();
 }
 
 void LevelBadge::updateText() {
-	auto text = _icon;
-	text.append(' ');
+	auto text = Ui::Text::IconEmoji(&st::settingsLevelBadgeLock).append(' ');
 	if (!_minimal) {
 		text.append(tr::lng_boost_level(
 			tr::now,
@@ -490,7 +477,7 @@ void LevelBadge::updateText() {
 		st,
 		text,
 		kMarkupTextOptions,
-		_context);
+		Ui::Text::MarkedContext{ .repaint = [=] { update(); } });
 	const auto &padding = st::settingsColorSamplePadding;
 	QWidget::resize(
 		_text.maxWidth() + rect::m::sum::h(padding),
@@ -1213,7 +1200,7 @@ void EditPeerColorBox(
 			peer,
 			state->index.value(),
 			state->emojiId.value()
-		), {});
+		), style::margins());
 
 		auto indices = peer->session().api().peerColors().suggestedValue();
 		const auto margin = st::settingsColorRadioMargin;
