@@ -110,6 +110,7 @@ struct StarGiftUpgradeArgs {
 	Fn<void(bool)> ready;
 	not_null<PeerData*> peer;
 	Data::SavedStarGiftId savedId;
+	QString giftPrepayUpgradeHash;
 	int cost = 0;
 	bool canAddSender = false;
 	bool canAddComment = false;
@@ -157,4 +158,18 @@ void ShowResaleGiftBoughtToast(
 	not_null<PeerData*> to,
 	const Data::UniqueGift &gift);
 
+[[nodiscard]] rpl::lifetime ShowStarGiftResale(
+	not_null<Window::SessionController*> controller,
+	not_null<PeerData*> peer,
+	uint64 giftId,
+	QString title,
+	Fn<void()> finishRequesting);
+
+[[nodiscard]] CreditsAmount StarsFromTon(
+	not_null<Main::Session*> session,
+	CreditsAmount ton);
+[[nodiscard]] CreditsAmount TonFromStars(
+	not_null<Main::Session*> session,
+	CreditsAmount stars);
+
 } // namespace Ui
