@@ -20,6 +20,10 @@ struct UniqueGift;
 class SavedStarGiftId;
 } // namespace Data
 
+namespace Ui {
+class ChatTheme;
+} // namespace Ui
+
 void ShowTransferToBox(
 	not_null<Window::SessionController*> controller,
 	not_null<PeerData*> peer,
@@ -45,3 +49,18 @@ bool ShowResaleGiftLater(
 bool ShowTransferGiftLater(
 	std::shared_ptr<ChatHelpers::Show> show,
 	std::shared_ptr<Data::UniqueGift> gift);
+
+void SetThemeFromUniqueGift(
+	not_null<Window::SessionController*> window,
+	std::shared_ptr<Data::UniqueGift> unique);
+void SendPeerThemeChangeRequest(
+	not_null<Window::SessionController*> controller,
+	not_null<PeerData*> peer,
+	const QString &token,
+	const std::shared_ptr<Data::UniqueGift> &unique,
+	bool locallySet = false);
+void SetPeerTheme(
+	not_null<Window::SessionController*> controller,
+	not_null<PeerData*> peer,
+	const QString &token,
+	const std::shared_ptr<Ui::ChatTheme> &theme);
