@@ -205,6 +205,7 @@ public:
 			Connecting,
 			AllowedToSpeak,
 			Ended,
+			RecordingStarted,
 		};
 		virtual void groupCallPlaySound(GroupCallSound sound) = 0;
 		virtual auto groupCallGetVideoCapture(const QString &deviceId)
@@ -283,6 +284,7 @@ public:
 		const QString &title,
 		bool video,
 		bool videoPortrait);
+	void playSoundRecordingStarted() const;
 	[[nodiscard]] bool recordingStoppedByMe() const {
 		return _recordingStoppedByMe;
 	}
