@@ -7,6 +7,9 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
+class QKeyEvent;
+class QShortcutEvent;
+
 namespace Shortcuts {
 
 enum class Command {
@@ -66,6 +69,9 @@ enum class Command {
 	SendSilentMessage,
 	ScheduleMessage,
 
+	RecordVoice,
+	RecordRound,
+
 	ReadChat,
 	ArchiveChat,
 
@@ -119,7 +125,7 @@ private:
 
 };
 
-rpl::producer<not_null<Request*>> Requests();
+[[nodiscard]] rpl::producer<not_null<Request*>> Requests();
 
 void Start();
 void Finish();
@@ -129,7 +135,15 @@ void Listen(not_null<QWidget*> widget);
 bool Launch(Command command);
 bool HandleEvent(not_null<QObject*> object, not_null<QShortcutEvent*> event);
 
-const QStringList &Errors();
+bool HandlePossibleChatSwitch(not_null<QKeyEvent*> event);
+
+struct ChatSwitchRequest {
+	Qt::Key action = Qt::Key_Tab; // Key_Tab, Key_Backtab or Key_Escape.
+	bool started = false;
+};
+[[nodiscard]] rpl::producer<ChatSwitchRequest> ChatSwitchRequests();
+
+[[nodiscard]] const QStringList &Errors();
 
 // Media shortcuts are not enabled by default, because other
 // applications also use them. They are enabled only when
