@@ -59,6 +59,7 @@ constexpr auto kRequestTimeLimit = 60 * crl::time(1000);
 			data.vid(),
 			data.vfrom_id() ? *data.vfrom_id() : MTPPeer(),
 			data.vpeer_id(),
+			data.vsaved_peer_id() ? *data.vsaved_peer_id() : MTPPeer(),
 			data.vreply_to() ? *data.vreply_to() : MTPMessageReplyHeader(),
 			data.vdate(),
 			data.vaction(),
@@ -96,7 +97,10 @@ constexpr auto kRequestTimeLimit = 60 * crl::time(1000);
 			MTP_long(data.veffect().value_or_empty()), // effect
 			data.vfactcheck() ? *data.vfactcheck() : MTPFactCheck(),
 			MTP_int(data.vreport_delivery_until_date().value_or_empty()),
-			MTP_long(data.vpaid_message_stars().value_or_empty()));
+			MTP_long(data.vpaid_message_stars().value_or_empty()),
+			(data.vsuggested_post()
+				? *data.vsuggested_post()
+				: MTPSuggestedPost()));
 	});
 }
 
@@ -271,7 +275,8 @@ void ScheduledMessages::sendNowSimpleMessage(
 			MTP_long(local->effectId()), // effect
 			MTPFactCheck(),
 			MTPint(), // report_delivery_until_date
-			MTPlong()), // paid_message_stars
+			MTPlong(), // paid_message_stars
+			MTPSuggestedPost()),
 		localFlags,
 		NewMessageType::Unread);
 
