@@ -66,6 +66,7 @@ class GroupCall;
 class NotifySettings;
 class CustomEmojiManager;
 class Stories;
+class SavedMusic;
 class SavedMessages;
 class Chatbots;
 class BusinessInfo;
@@ -184,6 +185,9 @@ public:
 	[[nodiscard]] Stories &stories() const {
 		return *_stories;
 	}
+	[[nodiscard]] SavedMusic &savedMusic() const {
+		return *_savedMusic;
+	}
 	[[nodiscard]] SavedMessages &savedMessages() const {
 		return *_savedMessages;
 	}
@@ -376,6 +380,11 @@ public:
 	void notifyPinnedDialogsOrderUpdated();
 	[[nodiscard]] rpl::producer<> pinnedDialogsOrderUpdated() const;
 
+	void nextForUpgradeGiftInvalidate(not_null<PeerData*> owner);
+	void nextForUpgradeGiftRequest(
+		not_null<PeerData*> owner,
+		Fn<void(std::optional<Data::SavedStarGift>)> done);
+
 	using CreditsSubsRebuilder = rpl::event_stream<CreditsStatusSlice>;
 	using CreditsSubsRebuilderPtr = std::shared_ptr<CreditsSubsRebuilder>;
 	[[nodiscard]] CreditsSubsRebuilderPtr createCreditsSubsRebuilder();
@@ -881,6 +890,13 @@ public:
 private:
 	using Messages = std::unordered_map<MsgId, not_null<HistoryItem*>>;
 
+	struct NextToUpgradeGift {
+		std::optional<Data::SavedStarGift> gift;
+		Fn<void(std::optional<Data::SavedStarGift>)> done;
+		crl::time received = 0;
+		mtpRequestId requestId = 0;
+	};
+
 	void suggestStartExport();
 
 	void setupMigrationViewer();
@@ -1223,6 +1239,7 @@ private:
 	const std::unique_ptr<NotifySettings> _notifySettings;
 	const std::unique_ptr<CustomEmojiManager> _customEmojiManager;
 	const std::unique_ptr<Stories> _stories;
+	const std::unique_ptr<SavedMusic> _savedMusic;
 	const std::unique_ptr<SavedMessages> _savedMessages;
 	const std::unique_ptr<Chatbots> _chatbots;
 	const std::unique_ptr<BusinessInfo> _businessInfo;
@@ -1232,6 +1249,10 @@ private:
 
 	std::unique_ptr<StarsRatingPending> _pendingStarsRating;
 
+	base::flat_map<
+		not_null<PeerData*>,
+		NextToUpgradeGift> _nextForUpgradeGifts;
+
 	rpl::lifetime _lifetime;
 
 };
