@@ -120,6 +120,10 @@ void MainList::unreadStateChanged(
 	const auto notify = !useClouded || wasState.known;
 	const auto notifier = unreadStateChangeNotifier(notify);
 	_unreadState += nowState - wasState;
+	if (_unreadState.chatsMuted > _unreadState.chats
+		|| _unreadState.messagesMuted > _unreadState.messages) {
+		[[maybe_unused]] int a = 0;
+	}
 	if (updateCloudUnread) {
 		Assert(nowState.known);
 		_cloudUnreadState += nowState - wasState;
@@ -145,6 +149,10 @@ void MainList::unreadEntryChanged(
 	} else {
 		_unreadState -= state;
 	}
+	if (_unreadState.chatsMuted > _unreadState.chats
+		|| _unreadState.messagesMuted > _unreadState.messages) {
+		[[maybe_unused]] int a = 0;
+	}
 	if (updateCloudUnread) {
 		if (added) {
 			_cloudUnreadState += state;
