@@ -80,6 +80,8 @@ public:
 	void monoforumChanged(Data::SavedMessages *old);
 	[[nodiscard]] bool amMonoforumAdmin() const;
 	[[nodiscard]] bool suggestDraftAllowed() const;
+	[[nodiscard]] bool hasForumThreadBars() const;
+	void forumTabsChanged(bool forumTabs);
 
 	[[nodiscard]] not_null<History*> migrateToOrMe() const;
 	[[nodiscard]] History *migrateFrom() const;
@@ -434,7 +436,10 @@ public:
 	void tryMarkMonoforumIntervalRead(
 		MsgId wasInboxReadBefore,
 		MsgId nowInboxReadBefore);
-	void validateMonoforumUnread(MsgId readTillId);
+	void tryMarkForumIntervalRead(
+		MsgId wasInboxReadBefore,
+		MsgId nowInboxReadBefore);
+	void validateMonoAndForumUnread(MsgId readTillId);
 
 	[[nodiscard]] bool isTopPromoted() const;
 
@@ -480,7 +485,7 @@ private:
 		FakeUnreadWhileOpened = (1 << 5),
 		HasPinnedMessages = (1 << 6),
 		ResolveChatListMessage = (1 << 7),
-		MonoforumUnreadInvalidatePending = (1 << 8),
+		MonoAndForumUnreadInvalidatePending = (1 << 8),
 	};
 	using Flags = base::flags<Flag>;
 	friend inline constexpr auto is_flag_type(Flag) {
