@@ -7,9 +7,11 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "info/channel_statistics/earn/earn_icons.h"
 
+#include "ui/effects/credits_graphics.h"
 #include "ui/effects/premium_graphics.h"
-#include "ui/text/text_custom_emoji.h"
+#include "ui/text/custom_emoji_instance.h"
 #include "ui/rect.h"
+#include "styles/style_credits.h"
 #include "styles/style_menu_icons.h"
 #include "styles/style_widgets.h"
 #include "styles/style_info.h" // infoIconReport.
@@ -144,9 +146,41 @@ QImage MenuIconCredits() {
 std::unique_ptr<Ui::Text::CustomEmoji> MakeCurrencyIconEmoji(
 		const style::font &font,
 		const QColor &c) {
-	return std::make_unique<Ui::Text::StaticCustomEmoji>(
-		IconCurrencyColored(font, c),
-		u"currency_icon:%1:%2"_q.arg(font->height).arg(c.name()));
+	return std::make_unique<Ui::CustomEmoji::Internal>(
+		u"currency_icon:%1:%2"_q.arg(font->height).arg(c.name()),
+		IconCurrencyColored(font, c));
+}
+
+Ui::Text::PaletteDependentEmoji IconCreditsEmoji(
+		IconDescriptor descriptor) {
+	return { .factory = [=] {
+		return Ui::GenerateStars(
+			(descriptor.size
+				? descriptor.size
+				: st::defaultTableLabel.style.font->height),
+			1);
+	}, .margin = descriptor.margin.value_or(
+		QMargins{ 0, st::giftBoxByStarsSkip, 0, 0 }) };
+}
+
+Ui::Text::PaletteDependentEmoji IconCurrencyEmoji(
+		IconDescriptor descriptor) {
+	return { .factory = [=] {
+		return IconCurrencyColored(
+			descriptor.size ? descriptor.size : st::earnTonIconSize,
+			st::currencyFg->c);
+	}, .margin = descriptor.margin.value_or(st::earnTonIconMargin) };
+}
+
+Ui::Text::PaletteDependentEmoji IconCreditsEmojiSmall() {
+	return IconCreditsEmoji({
+		.size = st::giftBoxByStarsStyle.font->height,
+		.margin = QMargins{ 0, st::giftBoxByStarsStarTop, 0, 0 },
+	});
+}
+
+Ui::Text::PaletteDependentEmoji IconCurrencyEmojiSmall() {
+	return IconCreditsEmoji({});
 }
 
 } // namespace Ui::Earn
