@@ -121,6 +121,14 @@ int AppConfig::pinnedGiftsLimit() const {
 	return get<int>(u"stargifts_pinned_to_top_limit"_q, 6);
 }
 
+int AppConfig::giftCollectionsLimit() const {
+	return get<int>(u"stargifts_collections_limit"_q, 10);
+}
+
+int AppConfig::giftCollectionGiftsLimit() const {
+	return get<int>(u"stargifts_collection_gifts_limit"_q, 500);
+}
+
 bool AppConfig::callsDisabledForSession() const {
 	const auto authorizations = _account->sessionExists()
 		? &_account->session().api().authorizations()
@@ -140,18 +148,32 @@ bool AppConfig::confcallPrioritizeVP8() const {
 	return get<bool>(u"confcall_use_vp8"_q, false);
 }
 
-int AppConfig::giftResalePriceMax() const {
-	return get<int>(u"stars_stargift_resale_amount_max"_q, 35000);
-}
-
-int AppConfig::giftResalePriceMin() const {
+int AppConfig::giftResaleStarsMin() const {
 	return get<int>(u"stars_stargift_resale_amount_min"_q, 125);
 }
 
-int AppConfig::giftResaleReceiveThousandths() const {
+int AppConfig::giftResaleStarsMax() const {
+	return get<int>(u"stars_stargift_resale_amount_max"_q, 35000);
+}
+
+int AppConfig::giftResaleStarsThousandths() const {
 	return get<int>(u"stars_stargift_resale_commission_permille"_q, 800);
 }
 
+int64 AppConfig::giftResaleNanoTonMin() const {
+	return get<int64>(u"ton_stargift_resale_amount_min"_q, 250'000'000LL);
+}
+
+int64 AppConfig::giftResaleNanoTonMax() const {
+	return get<int64>(
+		u"ton_stargift_resale_amount_max"_q,
+		1'000'000'000'000'000LL);
+}
+
+int AppConfig::giftResaleNanoTonThousandths() const {
+	return get<int>(u"ton_stargift_resale_commission_permille"_q, 800);
+}
+
 int AppConfig::pollOptionsLimit() const {
 	return get<int>(u"poll_answers_max"_q, 12);
 }
@@ -208,6 +230,30 @@ TimeId AppConfig::suggestedPostAgeMin() const {
 	return get<int>(u"stars_suggested_post_age_min"_q, 86400);
 }
 
+bool AppConfig::ageVerifyNeeded() const {
+	return get<bool>(u"need_age_video_verification"_q, false);
+}
+
+QString AppConfig::ageVerifyCountry() const {
+	return get<QString>(u"verify_age_country"_q, QString());
+}
+
+int AppConfig::ageVerifyMinAge() const {
+	return get<int>(u"verify_age_min"_q, 18);
+}
+
+QString AppConfig::ageVerifyBotUsername() const {
+	return get<QString>(u"verify_age_bot_username"_q, QString());
+}
+
+int AppConfig::storiesAlbumsLimit() const {
+	return get<int>(u"stories_albums_limit"_q, 100);
+}
+
+int AppConfig::storiesAlbumLimit() const {
+	return get<int>(u"stories_album_stories_limit"_q, 1000);
+}
+
 void AppConfig::refresh(bool force) {
 	if (_requestId || !_api) {
 		if (force) {
