@@ -33,12 +33,14 @@ namespace Data {
 class Session;
 class Changes;
 class RecentPeers;
+class RecentSharedMediaGifts;
 class ScheduledMessages;
 class SponsoredMessages;
 class TopPeers;
 class Factchecks;
 class LocationPickers;
 class Credits;
+class PromoSuggestions;
 } // namespace Data
 
 namespace HistoryView::Reactions {
@@ -80,6 +82,19 @@ class Domain;
 class SessionSettings;
 class SendAsPeers;
 
+struct FreezeInfo {
+	TimeId since = 0;
+	TimeId until = 0;
+	QString appealUrl;
+
+	explicit operator bool() const {
+		return since != 0;
+	}
+	friend inline bool operator==(
+		const FreezeInfo &,
+		const FreezeInfo &) = default;
+};
+
 class Session final : public base::has_weak_ptr {
 public:
 	Session(
@@ -119,6 +134,9 @@ public:
 	[[nodiscard]] Data::RecentPeers &recentPeers() const {
 		return *_recentPeers;
 	}
+	[[nodiscard]] Data::RecentSharedMediaGifts &recentSharedGifts() const {
+		return *_recentSharedGifts;
+	}
 	[[nodiscard]] Data::SponsoredMessages &sponsoredMessages() const {
 		return *_sponsoredMessages;
 	}
@@ -176,6 +194,9 @@ public:
 	[[nodiscard]] InlineBots::AttachWebView &attachWebView() const {
 		return *_attachWebView;
 	}
+	[[nodiscard]] Data::PromoSuggestions &promoSuggestions() const {
+		return *_promoSuggestions;
+	}
 	[[nodiscard]] auto cachedReactionIconFactory() const
 	-> HistoryView::Reactions::CachedIconFactory & {
 		return *_cachedReactionIconFactory;
@@ -236,12 +257,17 @@ public:
 	[[nodiscard]] Support::Templates &supportTemplates() const;
 	[[nodiscard]] Support::FastButtonsBots &fastButtonsBots() const;
 
+	[[nodiscard]] FreezeInfo frozen() const;
+	[[nodiscard]] rpl::producer<FreezeInfo> frozenValue() const;
+
 	[[nodiscard]] auto colorIndicesValue()
 		-> rpl::producer<Ui::ColorIndicesCompressed>;
 
 private:
 	static constexpr auto kDefaultSaveDelay = crl::time(1000);
 
+	void appConfigRefreshed();
+
 	const UserId _userId;
 	const not_null<Account*> _account;
 
@@ -265,6 +291,7 @@ private:
 	const std::unique_ptr<SendAsPeers> _sendAsPeers;
 	const std::unique_ptr<InlineBots::AttachWebView> _attachWebView;
 	const std::unique_ptr<Data::RecentPeers> _recentPeers;
+	const std::unique_ptr<Data::RecentSharedMediaGifts> _recentSharedGifts;
 	const std::unique_ptr<Data::ScheduledMessages> _scheduledMessages;
 	const std::unique_ptr<Data::SponsoredMessages> _sponsoredMessages;
 	const std::unique_ptr<Data::TopPeers> _topPeers;
@@ -272,6 +299,7 @@ private:
 	const std::unique_ptr<Data::Factchecks> _factchecks;
 	const std::unique_ptr<Data::LocationPickers> _locationPickers;
 	const std::unique_ptr<Data::Credits> _credits;
+	const std::unique_ptr<Data::PromoSuggestions> _promoSuggestions;
 
 	using ReactionIconFactory = HistoryView::Reactions::CachedIconFactory;
 	const std::unique_ptr<ReactionIconFactory> _cachedReactionIconFactory;
@@ -288,6 +316,8 @@ private:
 	base::flat_set<not_null<Window::SessionController*>> _windows;
 	base::Timer _saveSettingsTimer;
 
+	rpl::variable<FreezeInfo> _frozen;
+
 	QByteArray _tmpPassword;
 	TimeId _tmpPasswordValidUntil = 0;
 
