@@ -453,12 +453,18 @@ void SetupRows(
 
 	Ui::AddSkip(container);
 
+	const auto showEditName = [=] {
+		if (controller->showFrozenError()) {
+			return;
+		}
+		controller->show(Box<EditNameBox>(self));
+	};
 	AddRow(
 		container,
 		tr::lng_settings_name_label(),
 		Info::Profile::NameValue(self) | Ui::Text::ToWithEntities(),
 		tr::lng_profile_copy_fullname(tr::now),
-		[=] { controller->show(Box<EditNameBox>(self)); },
+		showEditName,
 		{ &st::menuIconProfile });
 
 	const auto showChangePhone = [=] {
@@ -508,6 +514,9 @@ void SetupRows(
 		std::move(value),
 		tr::lng_context_copy_mention(tr::now),
 		[=] {
+			if (controller->showFrozenError()) {
+				return;
+			}
 			const auto box = controller->show(
 				Box(UsernamesBox, session->user()));
 			box->boxClosing(
