@@ -106,6 +106,8 @@ public:
 
 	void createManager();
 	void setManager(Fn<std::unique_ptr<Manager>()> create);
+	[[nodiscard]] Manager &manager() const;
+	[[nodiscard]] rpl::producer<> managerChanged() const;
 
 	void checkDelayed();
 	void schedule(Data::ItemNotification notification);
@@ -124,7 +126,13 @@ public:
 	[[nodiscard]] rpl::producer<ChangeType> settingsChanged() const;
 	void notifySettingsChanged(ChangeType type);
 
-	void playSound(not_null<Main::Session*> session, DocumentId id);
+	[[nodiscard]] bool volumeSupported() const;
+	[[nodiscard]] rpl::producer<bool> volumeSupportedValue() const;
+
+	void playSound(
+		not_null<Main::Session*> session,
+		DocumentId id,
+		float64 volumeOverride = -1);
 	[[nodiscard]] QByteArray lookupSoundBytes(
 		not_null<Data::Session*> owner,
 		DocumentId id);
@@ -213,6 +221,7 @@ private:
 		crl::time> _sentReactionNotifications;
 
 	std::unique_ptr<Manager> _manager;
+	rpl::event_stream<> _managerChanged;
 
 	rpl::event_stream<ChangeType> _settingsChanged;
 
@@ -331,9 +340,7 @@ public:
 	[[nodiscard]] bool skipToast() const {
 		return doSkipToast();
 	}
-	void maybePlaySound(Fn<void()> playSound) {
-		doMaybePlaySound(std::move(playSound));
-	}
+	void maybePlaySound(Fn<void()> playSound);
 	void maybeFlashBounce(Fn<void()> flashBounce) {
 		doMaybeFlashBounce(std::move(flashBounce));
 	}
