@@ -270,5 +270,20 @@ TextWithEntities WrapEmailPattern(const QString &pattern) {
 	return { pattern };
 }
 
+QList<QStringView> Words(QStringView lower) {
+	static const auto kRegWords = QRegularExpression(
+		u"[\\W]"_q,
+		QRegularExpression::UseUnicodePropertiesOption);
+	return lower.split(kRegWords, Qt::SkipEmptyParts);
+}
+
+QString StripUrlProtocol(const QString &link) {
+	return link.startsWith(u"https://"_q)
+		? link.mid(8)
+		: link.startsWith(u"http://"_q)
+		? link.mid(7)
+		: link;
+}
+
 } // namespace Text
 } // namespace Ui
