@@ -12,17 +12,25 @@ namespace anim {
 enum class type : uchar;
 } // namespace anim
 
+namespace base {
+template <typename T>
+class unique_qptr;
+} // namespace base
+
 namespace style {
 struct PopupMenu;
 struct MenuSeparator;
 } // namespace style
 
 namespace Ui {
+class RpWidget;
 class PopupMenu;
 } // namespace Ui
 
 namespace Ui::Menu {
 
+class ItemBase;
+
 struct MenuCallback final {
 public:
 	struct Args {
@@ -31,6 +39,7 @@ public:
 		const style::icon *icon;
 		const style::MenuSeparator *separatorSt = nullptr;
 		FnMut<void(not_null<Ui::PopupMenu*>)> fillSubmenu;
+		FnMut<base::unique_qptr<ItemBase>(not_null<RpWidget*>)> make;
 		const style::PopupMenu *submenuSt = nullptr;
 		Fn<bool()> triggerFilter;
 		rpl::producer<anim::type> hideRequests;
